@@ -1,3 +1,11 @@
+# Copyright (c) 2019-2022, RTE (https://www.rte-france.com)
+# See AUTHORS.txt
+# This Source Code Form is subject to the terms of the Mozilla Public License, version 2.0.
+# If a copy of the Mozilla Public License, version 2.0 was not distributed with this file,
+# you can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-License-Identifier: MPL-2.0
+# This file is part of Chronix2Grid, A python package to generate "en-masse" chronics for loads and productions (thermal, renewable)
+
 import copy
 
 import numpy as np
@@ -7,23 +15,26 @@ from scipy.interpolate import interp1d
 from .. import generation_utils as utils
 import chronix2grid.constants as cst
 
-def compute_wind_series(locations, Pmax, long_noise, medium_noise, short_noise, params, smoothdist):
+def compute_wind_series(prng, locations, Pmax, long_noise, medium_noise, short_noise, params, smoothdist, add_dim):
     # Compute refined signals
     long_scale_signal = utils.interpolate_noise(
         long_noise,
         params,
         locations,
-        time_scale=params['long_wind_corr'])
+        time_scale=params['long_wind_corr'],
+        add_dim=add_dim)
     medium_scale_signal = utils.interpolate_noise(
         medium_noise,
         params,
         locations,
-        time_scale=params['medium_wind_corr'])
+        time_scale=params['medium_wind_corr'],
+        add_dim=add_dim)
     short_scale_signal = utils.interpolate_noise(
         short_noise,
         params,
         locations,
-        time_scale=params['short_wind_corr'])
+        time_scale=params['short_wind_corr'],
+        add_dim=add_dim)
 
     # Compute seasonal pattern
     Nt_inter = int(params['T'] // params['dt'] + 1)
@@ -39,34 +50,44 @@ def compute_wind_series(locations, Pmax, long_noise, medium_noise, short_noise,
     signal = (0.7 + 0.3 * seasonal_pattern) * (0.3 + std_medium_wind_noise * medium_scale_signal + std_long_wind_noise * long_scale_signal)
     signal += std_short_wind_noise * short_scale_signal
     signal = 1e-1 * np.exp(4 * signal)
-    # signal += np.random.uniform(0, SMOOTHDIST/Pmax, signal.shape)
-    signal += np.random.uniform(0, smoothdist, signal.shape)
+    #signal += prng.uniform(0, SMOOTHDIST/Pmax, signal.shape)
+    signal += prng.uniform(0, smoothdist, signal.shape)
+    #signal += np.random.uniform(0, smoothdist, signal.shape) #older version - to be removed
 
     # signal *= 0.95
     signal[signal < 0.] = 0.
     signal = smooth(signal)
     wind_series = Pmax * signal
-
+    wind_series[wind_series > 0.95 * Pmax] = 0.95 * Pmax
     return wind_series
 
-def compute_solar_series(locations, Pmax, solar_noise, params, solar_pattern, smoothdist, time_scale):
+def compute_solar_series(prng, locations, Pmax, solar_noise, params, solar_pattern, smoothdist, time_scale, add_dim, scale_solar_coord_for_correlation=None):
 
     # Compute noise at desired locations
-    final_noise = utils.interpolate_noise(solar_noise, params, locations, time_scale)
+    if scale_solar_coord_for_correlation is not None:
+        locations = [float(scale_solar_coord_for_correlation) * float(locations[0]), float(scale_solar_coord_for_correlation) * float(locations[1])]
+    final_noise = utils.interpolate_noise(solar_noise, params, locations, time_scale, add_dim=add_dim)
 
     # Compute solar pattern
     solar_pattern = compute_solar_pattern(params, solar_pattern)
 
     # Compute solar time series
     std_solar_noise = float(params['std_solar_noise'])
-    signal = solar_pattern*(0.75+std_solar_noise*final_noise)
-    signal += np.random.uniform(0, smoothdist/Pmax, signal.shape)
+    if "mean_solar_pattern" in params:
+        mean_solar_pattern = float(params["mean_solar_pattern"])
+    else:
+        # legacy behaviour
+        mean_solar_pattern = 0.75
+        
+    signal = solar_pattern * (mean_solar_pattern + std_solar_noise * final_noise)
+    #signal += prng.uniform(0, smoothdist/Pmax, signal.shape) #to be revised: since smmothdist/PMax is very small, the added noise compared to the previous sinal was unsignificant
+    #signal += np.random.uniform(0, smoothdist / Pmax, signal.shape) #older version - to be removed
     # signal[signal > 1] = 1
     signal[signal < 0.] = 0.
     signal = smooth(signal)
-    solar_series = Pmax*signal
+    solar_series = Pmax * signal
     # solar_series[np.isclose(solar_series, 0.)] = 0
-
+    solar_series[solar_series > 0.95 * Pmax] = 0.95 * Pmax
     return solar_series
 
 def compute_solar_pattern(params, solar_pattern):
@@ -141,7 +162,7 @@ def smooth(x, alpha=0.5, beta=None):
     return x
 
 
-def create_csv(dict_, path, reordering=True, noise=None, shift=False,
+def create_csv(prng, dict_, path, reordering=True, noise=None, shift=False,
                write_results=True, index=False):
     if type(dict_) is dict:
         df = pd.DataFrame.from_dict(dict_)
@@ -157,7 +178,8 @@ def create_csv(dict_, path, reordering=True, noise=None, shift=False,
         new_ordering = [x for _ ,x in sorted(zip(value ,list(df)))]
         df = df[new_ordering]
     if noise is not None:
-        df *= ( 1 +noise *np.random.normal(0, 1, df.shape))
+        df *= ( 1 +noise * prng.normal(0, 1, df.shape))
+        #df *= (1 + noise * np.random.normal(0, 1, df.shape)) #older version - to be removed
     if shift:
         df = df.shift(-1)
         df = df.fillna(0)
