@@ -1,3 +1,11 @@
+# Copyright (c) 2019-2022, RTE (https://www.rte-france.com)
+# See AUTHORS.txt
+# This Source Code Form is subject to the terms of the Mozilla Public License, version 2.0.
+# If a copy of the Mozilla Public License, version 2.0 was not distributed with this file,
+# you can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-License-Identifier: MPL-2.0
+# This file is part of Chronix2Grid, A python package to generate "en-masse" chronics for loads and productions (thermal, renewable)
+
 # Python built-in modules
 import warnings
 import os
@@ -107,6 +115,11 @@ def main(kpi_input_folder, generation_output_folder, scenario_names,
         dispatch_validator.plot_carriers_pw(curve='synthetic', stacked=True, max_col_splot=max_col, save_html=True,
                                             wind_solar_only=wind_solar_only)
 
+        dispatch_validator.plot_carriers_pw(curve='reference', stacked=False, max_col_splot=max_col, save_html=True,
+                                            wind_solar_only=wind_solar_only)
+        dispatch_validator.plot_carriers_pw(curve='synthetic', stacked=False, max_col_splot=max_col, save_html=True,
+                                            wind_solar_only=wind_solar_only)
+
         # Get Load KPI
         dispatch_validator.load_kpi()
 
